#include <reg164ci.h>

extern unsigned int *id_ptr_16x[16];		/* pointer to message id's (UAReg) */
extern unsigned char *db0_ptr_16x[16];		/* pointer to 'databyte 0's */	
extern unsigned int *msgctrl_ptr_16x[16];	/* ptr to msg. contrl. regs */
extern unsigned char *msgconf_ptr_16x[16];	/* ptr to msg. conf. regs */
extern unsigned char dir_bit_16x[16];		/* DIR bits MO 1...15 */
extern unsigned char xtd_bit_16x[16];		/* XTD bits MO 1...15 */
extern unsigned char dlc_16x[16];			/* data byte lengths MO 1...15 */

void def_mo_16x(unsigned char nr, unsigned char xtd, unsigned long id,
	unsigned char dir, unsigned char dlc, unsigned char txie,
	unsigned char rxie) {

/* Define message object: MO-number, xtd, id, dir, dlc, TXIE, RXIE */
/* -------------------------------- local  byte/word/bit data: -- */
	unsigned int dummy_int;
	unsigned int *dummy_idptr;

/* -------------------------------- program: -------------------- */
	if ((nr < 16) && (nr)) {
		dummy_idptr = id_ptr_16x [nr] + 1;	/* set dummy ptr to LAR */

	   	if (xtd) {		// load Arbitration Registers with XTD ID:
						// load Upper Arb. Reg.:
			id = id << 3;
			dummy_int = (unsigned int) (id >> 16);
			*id_ptr_16x [nr] = (dummy_int << 8) + (dummy_int >> 8); 
/* load Lower Arb. Reg. */
			dummy_int = (unsigned int) id;
			*dummy_idptr = (dummy_int << 8) + (dummy_int >> 8);
	   } else {		/* load Arbitration Registers with STD ID: */
/* load Upper Arb. Reg.: */
			dummy_int = (unsigned int) id;
			dummy_int = dummy_int << 5;
			*id_ptr_16x [nr] = (dummy_int << 8) + (dummy_int >> 8);
/* load Lower Arb. Reg.: */
			*dummy_idptr=0x0800;
	   }

	   /* prepare Message Control Register: */
		if (txie == 1)
	   		(txie = 0x20);
	   	else (txie = 0x10);
	   	if (rxie == 1)
			(rxie = 0x08);
	   	else (rxie = 0x04);
	   	if (dir == 1)
			dummy_int = (0x5981 | txie | rxie); /* CPUUPD set */
	   	else dummy_int = (0x5581 | txie | rxie);	   /* MSGLST reset */

	   	*msgctrl_ptr_16x[nr]=dummy_int;	/* load Mess. Contr. Reg. */

	   /* prepare Message Configuration Register: */
	   	if (dlc>8) dlc=8;
	   	dlc_16x[nr]=dlc;
	   	dir_bit_16x[nr]=dir;
	   	xtd_bit_16x[nr]=xtd;
	   	*msgconf_ptr_16x[nr] = (dlc<<4) + (dir<<3) + (xtd<<2);
	}

/* The function def_mo_16x defines a message object for the CAN 164 module
with a standard or extended identifier.
The function parameters are :
- Parameter nr defines the number of the message object (1..15)
- Parameter xtd defines if it is a 11-bit id (xtd=0) or 29-bit id (xtd=1)
- Parameter id defines the 11/29-Bit identifier of the message object
- Parameter dir (direction) defines if the message object is a receive object
  (dir=0) or a transmit object (dir=1)
- Parameter dlc (data length code) defines the data length
  of the message object (0 data bytes ... 8 data bytes)
- Parameter txie enables transmit interrupts of this MO
- Parameter rxie enables receive interrupts of this MO

Other Message control register loading ( t r a n s m i t  object):

         5                9               ?             ? 
    01       01       10      01      10    user   user     01
  RMTPND    TXRQ    CPUUPD  NEWDAT  MSGVAL  TXIE   RXIE   INTPND
   15 14    13 12    11 10    9 8    7 6     5 4    3 2    1 0

Other Message control register loading ( r e c e i v e  object):

         5                5               ?             ? 
    01       01       01      01      10    user   user     01
  RMTPND    TXRQ    MSGLST  NEWDAT  MSGVAL  TXIE   RXIE   INTPND
   15 14    13 12    11 10    9 8    7 6     5 4    3 2    1 0
*/
}

void ld_modata_16x(unsigned char nr, unsigned char *upl_data_ptr)
	/* load data bytes of message object 'nr' (1..14) */
{
   /* -------------------------------- local  byte/word/bit data: -- */
	unsigned char i;
	unsigned char *dummy_dbptr;
   /* -------------------------------- program: -------------------- */
	if ((nr<15) && (nr))
	{
	   	*msgctrl_ptr_16x[nr]=0xfaff;		/* set CPUUPD, NEWDAT */
	   	dummy_dbptr=db0_ptr_16x[nr];		/* load dummy ptr (db 0) */
	   	for (i=0;i<dlc_16x[nr];i++)
			*dummy_dbptr++ = *upl_data_ptr++;
/* move data bytes from upload buffer to MO data bytes */
	   	*msgctrl_ptr_16x[nr]=0xf7ff;
/* reset CPUUPD (NEWDAT is reset by CAN Module on Transm.) */
	}
}

void send_mo_16x(unsigned char nr)
	/* Transmit message object 'nr' (1..14) */
{
   /* -------------------------------- local  byte/word/bit data: -- */
   /* -------------------------------- program: -------------------- */
	if ((nr<15) && (nr))
		*msgctrl_ptr_16x[nr]=0xefff;  /* set TXRQ */
   /* -------------------------------- end: ------------------------ */
}
