#include <reg164ci.h>
#include "can_init.h"

#define T78M		0		// Timer-Mode
#define T7I			6		// Prescaler = 8
#define CPUCLOCK	24000000
#define PRESCALER	(1 << (T7I + 3))
#define FREQ		1		// Ausgangsfrequenz 1Hz TP 1s
#define VT			(0xffff) - (CPUCLOCK / PRESCALER / FREQ)

#pragma global
/* C164CI variables: */
unsigned int *id_ptr_16x[16];		/* pointer to message id's (UAReg) */
unsigned char *db0_ptr_16x[16];		/* pointer to 'databyte 0's */	
unsigned int *msgctrl_ptr_16x[16];	/* pointer to msg. contrl. regs */
unsigned char *msgconf_ptr_16x[16];	/* pointer to msg. conf. regs */
unsigned char dir_bit_16x[16];		/* DIR bits MO 1...15 */
unsigned char xtd_bit_16x[16];		/* XTD bits MO 1...15 */
unsigned char dlc_16x[16];		/* data byte lengths MO 1...15 */
#pragma public

/* ----------------------------------- code ------------------------ */
void init_can_16x(unsigned int baud_rate, unsigned char eie,
	unsigned char sie, unsigned char ie)
	/* Initialization of CAN-Module: (baud rate, eie, sie, ie) */
{       
   /* -------------------------------- local  byte/word/bit data: -- */
	unsigned char i, n;
	unsigned char *dummy_dbptr;
   /* -------------------------------- program: -------------------- */
	/* Initialization PORT4 (CAN) (P4.6 to output; P4.5 to input): */
	_bfld (P4, 0x0060, 0x0060);
	_bfld (DP4, 0x0060, 0x0040);

/* Load C164 pointers: */
	for (i=1; i<16; i++)
	{
	   db0_ptr_16x[i] = (unsigned char *)(0xef07+i*16);
			/* set pointers to data bytes 0 of MO 1..15 */
	   id_ptr_16x[i] = (unsigned int *)(0xef02+i*16);
			/* set pointers to id's of MO 1..15 (UARs) */
	   msgconf_ptr_16x[i] = (unsigned char *)(0xef06+i*16);
		/* set pointers to Message Conf. Registers of MO 1..15 */
	   msgctrl_ptr_16x[i] = (unsigned int *)(0xef00+i*16);
		/* set pointers to Message Control Registers of MO 1..15 */
	   dir_bit_16x[i] = 0;		/* clear DIR bit array */
	   xtd_bit_16x[i] = 0;		/* clear XTD bit array */
	   dlc_16x[i] = 0;		/* clear data length code array */
	}

/* Load General CAN-Registers: */
	CR = 0x41;	/* set CCE and INIT in Control Register (EF00h) */
	SR = 0x00;	/* Clear Status Partition (EF01h) */

	switch (baud_rate) {
		case 20: BTR = CAN_BTR_20KB;
			break;		
		case 50:	BTR = CAN_BTR_50KB;
			break;
	   	case 125:	BTR = CAN_BTR_125KB;
			break;
	   	case 250:	BTR = CAN_BTR_250KB;
			break;
	   	case 500:	BTR = CAN_BTR_500KB;
			break;
	   	case 1000:	BTR = CAN_BTR_1000KB;
			break;
	   	default:	BTR = CAN_BTR_125KB;
			break;
	}
	P1L = baud_rate;

	GMS=0xe0ff;	/* Global Mask Short (EF06h) */
			/* each bit of standard ID must match to store mess. */

	UGML=0xffff;	/* Upper Global Mask Long (EF08h) */
	LGML=0xf8ff;	/* Lower Global Mask Long (EF0Ah) */
			/* each bit of extended ID must match to store mess. */

	UMLM=0x0000;	/* Upper Mask of Last Message (EF0Ch) */
	LMLM=0x0000;	/* Lower Mask of Last Message (EF0Eh) */
			/* every message into MO 15 (Basic CAN Feature)*/

	/* reset all elements incl MSGVAL in all Message Object Ctrl. Reg.: */
	for (i=1; i<16; i++) *msgctrl_ptr_16x[i] = 0x5555;

	/* reset all data bytes in all Message Objects: */
	for (i=1; i<16; i++)
	{
		dummy_dbptr=db0_ptr_16x[i];	
	   	for (n=0; n<8; n++) *dummy_dbptr++ = 0x00;
	}

	/* end initialization (CCE=0, INIT=0); Interrupts EIE, SIE, IE=user: */
	CR = (0x00 | (eie<<3) | (sie<<2) | (ie<<1));
}

void T78_Init (void) {
	T78CON |= ((T78M<<3) | T7I);
	T7IC = 0x0012;
	T7 = VT;			// Teilerfaktor fr 250s Interrupt
	T7REL = VT;
	T7IE = 1;
}
