#include <reg164ci.h>

#define MY_IEN_BIT	1	/* General Interrupt Enable = yes */
#define MY_XP0IC_VALUE	0x00	/* Interrupts from CAN mod. (CPU side) = no */

#define MY_BAUD_RATE	125		/* 125 kBit/s on the CAN bus */ 
#define IE_BIT			0		/* Interrupts from CAN mod. (CAN side) = no */
#define EIE_BIT			0		/* no error interrupts from the CAN module */ 
#define SIE_BIT			0		/* no status interrupts from the CAN module */

#define TEST_IDENTIFIER 0x15abcdef	/* Test ID used by example program */

#define MO1_XTD_BIT		0		/* MO1 uses 11 bit ID */
#define MO1_ID			0x001	/* 11-bit identifier of MO1 */
#define MO1_DIR_BIT		1		/* MO1 transmits DF, receives RF */
#define MO1_DLC			8		/* MO1 data length = 8 bytes */
#define MO1_TXIE_BIT	0		/* MO1 gen. no transmit interrupts */
#define MO1_RXIE_BIT	0		/* MO1 gen. no receive interrupts */

#define MO2_XTD_BIT		0		/* MO1 uses 11 bit ID */
#define MO2_ID			0x002	/* 11-bit identifier of MO1 */
#define MO2_DIR_BIT		1		/* MO1 transmits DF, receives RF */
#define MO2_DLC			8		/* MO1 data length = 8 bytes */
#define MO2_TXIE_BIT	0		/* MO1 gen. no transmit interrupts */
#define MO2_RXIE_BIT	0		/* MO1 gen. no receive interrupts */

// Prototypen
extern void CAN_Init (void);
extern void T78_Init (void);

extern void init_can_16x(unsigned int baud_rate, unsigned char eie,
	unsigned char sie, unsigned char ie);
	/* Initialization of CAN-Module: (baud_rate, eie, sie, ie) */
extern void def_mo_16x(unsigned char nr, unsigned char xtd, unsigned long id,
	unsigned char dir, unsigned char dlc, unsigned char txie,
	unsigned char rxie);
	/* Define message object: MO-number, xtd, id, dir, dlc, TXIE, RXIE */
extern void ld_modata_16x(unsigned char nr, unsigned char *upl_data_ptr);
	/* load data bytes of a message object (1..14) */
extern void send_mo_16x(unsigned char nr);
	/* Transmit message object 'nr' (1..14) */

// globale Variablen
//unsigned char *dptr;
//unsigned int MyIdent;

unsigned char receive_buffer[8];
unsigned char upload_data_buf[8] = {0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77};
//unsigned char upload_data_buf [8] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};

void main (void) {
		
    init_can_16x(MY_BAUD_RATE, EIE_BIT, SIE_BIT, IE_BIT);
	/* Initialization of CAN-Module: (baud_rate, eie, sie, ie) */
	
	T78_Init ();
		
	DP3 |= 0x2000;
	ODP3 |= 0x2000;
	DP1L = 0xff;

	T7R = 1;
	IEN = 1;
	
//	MyIdent = 0x0765;	
		
	def_mo_16x (1, MO1_XTD_BIT, MO1_ID, MO1_DIR_BIT, \
				MO1_DLC, MO1_TXIE_BIT, MO1_RXIE_BIT);

	def_mo_16x (2, MO2_XTD_BIT, MO2_ID, MO2_DIR_BIT, \
				MO2_DLC, MO2_TXIE_BIT, MO2_RXIE_BIT);

	while (1) {
	}
}

interrupt (0x3d)	// Timer Interrupt T2INT
void T7_Int (void) {
	unsigned char i;
#pragma asm
	PUSH DPP1
	MOV	DPP1,#PAG ?BASE_DPP1
#pragma endasm

	ld_modata_16x (1, upload_data_buf);

	send_mo_16x (1);

	for (i=0; i<8; i++)
		upload_data_buf [i]++;
	
//	P1L = upload_data_buf [5];

	ld_modata_16x (2, upload_data_buf);

	send_mo_16x (2);
	
//	if (count++ == 10) {
		SCLK = ~SCLK;
//		count = 0;
//	}
	
#pragma asm
	POP DPP1
#pragma endasm

}
